=== modified file 'breezy/git/tests/test_blackbox.py'
--- breezy/git/tests/test_blackbox.py	2018-12-15 14:21:37 +0000
+++ breezy/git/tests/test_blackbox.py	2018-12-18 19:51:52 +0000
@@ -395,6 +395,19 @@
         self.assertEqual(error, '')
 
 
+class StatusTests(ExternalBase):
+
+    def test_empty_dir(self):
+        tree = self.make_branch_and_tree('.', format='git')
+        self.build_tree(['a/', 'a/foo'])
+        self.build_tree_contents([('.gitignore', 'foo\n')])
+        tree.add(['.gitignore'])
+        tree.commit('add ignore')
+        output, error = self.run_bzr('st')
+        self.assertEqual(output, '')
+        self.assertEqual(error, '')
+
+
 class StatsTests(ExternalBase):
 
     def test_simple_stats(self):

=== modified file 'breezy/git/tree.py'
--- breezy/git/tree.py	2018-11-22 03:51:03 +0000
+++ breezy/git/tree.py	2018-12-11 17:29:18 +0000
@@ -679,6 +679,7 @@
     if target_extras is None:
         target_extras = set()
     ret = delta.TreeDelta()
+    added = []
     for (oldpath, newpath), (oldmode, newmode), (oldsha, newsha) in changes:
         if newpath == b'' and not include_root:
             continue
@@ -705,14 +706,7 @@
         if oldpath is None and newpath is None:
             continue
         if oldpath is None:
-            if newpath in target_extras:
-                ret.unversioned.append(
-                    (osutils.normalized_filename(newpath)[0], None,
-                     mode_kind(newmode)))
-            else:
-                file_id = new_fileid_map.lookup_file_id(newpath_decoded)
-                ret.added.append(
-                    (newpath_decoded, file_id, mode_kind(newmode)))
+            added.append((newpath, mode_kind(newmode)))
         elif newpath is None or newmode == 0:
             file_id = old_fileid_map.lookup_file_id(oldpath_decoded)
             ret.removed.append((oldpath_decoded, file_id, mode_kind(oldmode)))
@@ -739,6 +733,22 @@
             ret.unchanged.append(
                 (newpath_decoded, file_id, mode_kind(newmode)))
 
+    implicit_dirs = {b''}
+    for path, kind in added:
+        if kind == 'directory' or path in target_extras:
+            continue
+        implicit_dirs.update(osutils.parent_directories(path))
+
+    for path, kind in added:
+        if kind == 'directory' and path not in implicit_dirs:
+            continue
+        path_decoded = osutils.normalized_filename(path)[0]
+        if path in target_extras:
+            ret.unversioned.append((path_decoded, None, kind))
+        else:
+            file_id = new_fileid_map.lookup_file_id(path_decoded)
+            ret.added.append((path_decoded, file_id, kind))
+
     return ret
 
 

=== modified file 'breezy/status.py'
--- breezy/status.py	2018-11-17 16:53:10 +0000
+++ breezy/status.py	2018-12-06 13:38:49 +0000
@@ -150,16 +150,18 @@
                 new = wt
         with old.lock_read(), new.lock_read():
             for hook in hooks['pre_status']:
-                hook(StatusHookParams(old, new, to_file, versioned,
-                                      show_ids, short, verbose, specific_files=specific_files))
+                hook(StatusHookParams(
+                    old, new, to_file, versioned, show_ids, short, verbose,
+                    specific_files=specific_files))
 
             specific_files, nonexistents \
                 = _filter_nonexistent(specific_files, old, new)
             want_unversioned = not versioned
 
             # Reporter used for short outputs
-            reporter = _mod_delta._ChangeReporter(output_file=to_file,
-                                                  unversioned_filter=new.is_ignored, classify=classify)
+            reporter = _mod_delta._ChangeReporter(
+                output_file=to_file, unversioned_filter=new.is_ignored,
+                classify=classify)
             report_changes(to_file, old, new, specific_files,
                            reporter, show_long_callback,
                            short=short, want_unversioned=want_unversioned,
@@ -183,8 +185,8 @@
             # delta.
             conflicts = new.conflicts()
             if specific_files is not None:
-                conflicts = conflicts.select_conflicts(new, specific_files,
-                                                       ignore_misses=True, recurse=True)[1]
+                conflicts = conflicts.select_conflicts(
+                    new, specific_files, ignore_misses=True, recurse=True)[1]
             if len(conflicts) > 0 and not short:
                 to_file.write("conflicts:\n")
             for conflict in conflicts:
@@ -214,8 +216,9 @@
             if nonexistents:
                 raise errors.PathsDoNotExist(nonexistents)
             for hook in hooks['post_status']:
-                hook(StatusHookParams(old, new, to_file, versioned,
-                                      show_ids, short, verbose, specific_files=specific_files))
+                hook(StatusHookParams(
+                    old, new, to_file, versioned, show_ids, short, verbose,
+                    specific_files=specific_files))
 
 
 def _get_sorted_revisions(tip_revision, revision_ids, parent_map):

